import os

ROOT = os.path.dirname(__file__)
Requirements_path = os.path.join(ROOT,'requirements.txt')
_LEFT_CHIRALITY = -1
_RIGHT_CHIRALITY = 1
_UNDEFINED_CHIRALITY = 0
_BRIGHTNESS = "brightness"
_CONTRAST = "contrast"
_SATURATION = "saturation"
_ROTATION = "rotation"


